/*	MikMod sound library
	(c) 1998, 1999, 2000 Miodrag Vallat and others - see file AUTHORS for
	complete list.

	This library is DFREE software; you can redistribute it and/or modify
	it under the terms of the GNU Library General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

/*==============================================================================

  $Id: drv_win.c,v 1.5 2002/01/08 21:00:53 miod Exp $

  Driver for output on win32 platforms using the multimedia API

==============================================================================*/

/*

	Written by Bjornar Henden <bhenden@online.no>

*/
#define GP32 1

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "mikmod_internals.h"

#ifdef GP32

#define GP32_buffersize 2048
#define GP32_ringsize GP32_buffersize*2

static unsigned char *GP32_buffer=NULL;

static BOOL GP32_IsThere(void){
	return 1;
}

static BOOL GP32_Init(void){
	md_mode = DMODE_STEREO | DMODE_16BITS | DMODE_SOFT_MUSIC | DMODE_SOFT_SNDFX;
	md_mixfreq = 44100;
	gp_initSound( 44100, 16, GP32_ringsize ); // 44k sound, 16bpp, 2x4k buffers
	GP32_buffer = malloc( GP32_buffersize ); // Half of the 8k ringbuffer
	gp_clearRingbuffer();
	return VC_Init();
}

void GP32_Stop(void){
	gp_clearRingbuffer();
}

void GP32_Restart(void){
	// No restart yet
}

static void GP32_Exit(void){
	gp_clearRingbuffer();
	free( GP32_buffer );
	VC_Exit();
}

static void GP32_Update(void){
	unsigned long bytesread;
	
#ifdef GP32_DEBUG
	//gp_debug( 1, "Read bytes" );
#endif
	
	bytesread = VC_WriteBytes(GP32_buffer, GP32_buffersize);	
	
#ifdef GP32_DEBUG
	//gp_debug( 1, "Read %d bytes", bytesread );
#endif
		
	if(!bytesread) return; // exit if a whole buffer hasn't been read
		
	gp_addRingsegment( (unsigned short*)GP32_buffer );
}

static void GP32_PlayStop(void){
	gp_clearRingbuffer();
	VC_PlayStop();
}

MIKMODAPI MDRIVER drv_gp32={
	NULL,
	"GP32 SDK Audio v0.1",
	"GP32 SDK Audio driver v0.1",
	0,255,
	"gp32",

	NULL,
	NULL,
	GP32_IsThere,
	VC_SampleLoad,
	VC_SampleUnload,
	VC_SampleSpace,
	VC_SampleLength,
	GP32_Init,
	GP32_Exit,
	NULL,
	VC_SetNumVoices,
	VC_PlayStart,
	GP32_PlayStop,
	GP32_Update,
	NULL,
	VC_VoiceSetVolume,
	VC_VoiceGetVolume,
	VC_VoiceSetFrequency,
	VC_VoiceGetFrequency,
	VC_VoiceSetPanning,
	VC_VoiceGetPanning,
	VC_VoicePlay,
	VC_VoiceStop,
	VC_VoiceStopped,
	VC_VoiceGetPosition,
	VC_VoiceRealVolume
};

#else

MISSING(drv_gp32);

#endif

/* ex:set ts=4: */
